/*
  Project: Animatronic Skull with Arduino
  Date: July 28, 2023
  Autor: Laboratório Maker
  Canal no YouTube: https://bit.ly/laboratoriomaker
  Blog: https://laboratoriomaker.blogspot.com
  Instagram: https://www.instagram.com/laboratoriomaker

  Description:
  In this project, we created an animated skull using Arduino and some electronic components.
  The skull has two red LEDs in the eyes and three micro servo motors,
  with two of them responsible for eye movements and the other for jaw movement.

  An ultrasonic sensor (HC-SR04) is used to detect the proximity of objects
  in relation to the skull. As a person approaches or moves away from the skull,
  different animations are activated. Eye and jaw animations vary in speed and intensity,
  providing a fun and interactive experience.

  This code was developed for educational and entertainment purposes.
  Visit the YouTube channel, blog, and Instagram for more projects and content related
  to electronics, programming, and Maker activities.
*/


#include <Servo.h>  // Library for controlling servos
#include <EEPROM.h> // Library for using EEPROM memory
#include <HCSR04.h> // Library for controlling the ultrasonic sensor

const int trigPin = 8;       // Trigger pin of the ultrasonic sensor
const int echoPin = 7;       // Echo pin of the ultrasonic sensor

const int leftEyePin = 3;    // Control pin for the left eye servo
const int rightEyePin = 5;   // Control pin for the right eye servo
const int jawPin = 11;       // Control pin for the jaw servo

Servo leftEye;       // Object to control the left eye servo
Servo rightEye;      // Object to control the right eye servo
Servo jaw;           // Object to control the jaw servo

const int leftEyeOut = 50;      // Position of the left eye to look outward
const int leftEyeIn = 130;      // Position of the left eye to look inward
int leftEyeFront = 90;          // Initial position of the left eye (front)

const int rightEyeOut = 130;    // Position of the right eye to look outward
const int rightEyeIn = 50;      // Position of the right eye to look inward
int rightEyeFront = 90;         // Initial position of the right eye (front)

int closed = 140;          // Position of the closed jaw
const int opening1 = 110;  // First opening position of the jaw
const int opening2 = 90;   // Second opening position of the jaw
const int opening3 = 70;   // Third opening position of the jaw

const int distanceFar = 50;   // Distance above 50 centimeters (for far animation)
const int distanceMedium = 30;   // Distance between 30 and 50 centimeters (for medium animation)
const int distanceClose = 15;   // Distance less than or equal to 15 centimeters (for close animation)

float distance;         // Variable to store the distance measured by the ultrasonic sensor

HCSR04 hc(trigPin, echoPin);  // Object to control the ultrasonic sensor

void setup() {
  Serial.begin(9600); // Initialize serial communication (for debugging)

  leftEye.attach(leftEyePin);   // Attach the object to the left eye servo pin
  rightEye.attach(rightEyePin); // Attach the object to the right eye servo pin
  jaw.attach(jawPin);           // Attach the object to the jaw servo pin

  leftEye.write(leftEyeFront);  // Set the initial position of the left eye (front)
  rightEye.write(rightEyeFront);// Set the initial position of the right eye (front)
  jaw.write(closed);            // Close the jaw initially
}

void loop() {
  float totalDistance = 0;

  for (int i = 0; i < 5; i++) {
    totalDistance += hc.dist();   // Take 5 consecutive distance measurements and sum the values
    delay(100);                   // Wait for a short interval between each measurement
  }

  distance = totalDistance / 5;  // Calculate the average of the 5 measurements for a more stable distance

  if (distance > distanceFar) {
    farAnimation();  // Call the function for animation when the distance is "far"
  } else if (distance > distanceMedium) {
    mediumAnimation();  // Call the function for animation when the distance is "medium"
  } else if (distance > distanceClose) {
    closeAnimation();  // Call the function for animation when the distance is "close"
  } else {
    veryCloseAnimation(); // Call the function for animation when the distance is "very close"
  }
}

void farAnimation() {
  int eyePosition = random(leftEyeOut - 10, leftEyeIn + 10);
  leftEye.write(eyePosition);  // Move the eyes to a random position
  rightEye.write(eyePosition);
  delay(random(500, 1000));   // Wait for a random time to create a blinking effect
}

void mediumAnimation() {
  int speed = random(1, 4); // 1: fast, 2: medium, 3: slow

  for (int i = 0; i < 30; i++) { // Smooth eye movement
    int eyePosition = random(leftEyeIn + 10, leftEyeOut - 10);
    leftEye.write(eyePosition);  // Move the eyes to a random position
    rightEye.write(eyePosition);
    delay(30 * speed);  // Wait for a short interval between each eye movement
  }

  int jawPosition = random(opening2, opening3);
  jaw.write(jawPosition); // Open the jaw to a random position
  delay(300);
  jaw.write(closed);    // Close the jaw again after a short interval
  delay(300);
}

void closeAnimation() {
  int speed = random(1, 4); // Set the speed of eye and jaw movement (1: fast, 2: medium, 3: slow)

  for (int i = 0; i < 30; i++) { // Loop for smooth eye movement
    int eyePosition = random(leftEyeIn + 10, leftEyeOut - 10);
    leftEye.write(eyePosition); // Move the eyes to a random position
    rightEye.write(eyePosition);
    delay(30 * speed); // Wait for a short interval between each eye movement
  }

  int jawPosition = random(opening2, opening3); // Set a random position for the jaw
  jaw.write(jawPosition); // Open the jaw to the random position
  delay(300); // Wait for a short interval before closing the jaw again
  jaw.write(closed); // Close the jaw
  delay(300); // Wait for another short interval
}

void veryCloseAnimation() {
  leftEye.write(leftEyeIn); // Move the left eye inward (cross-eyed)
  rightEye.write(rightEyeIn); // Move the right eye inward (cross-eyed)
  delay(1000); // Wait for a second

  // Quick laughter - Jaw chattering
  for (int i = 0; i < 5; i++) { // Loop for quick laughter
    jaw.write(opening1); // Open the jaw
    delay(100); // Wait for a short interval
    jaw.write(closed); // Close the jaw
    delay(100); // Wait for another short interval
  }
  delay(500); // Wait for half a second after the laughter
}
